

% evacuation (script)
% Modelliert die Bewegung von Menschen, die auf eine Tr zustrmen

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                  Dies ist das Hauptprogramm!!!!!!!!!                   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Block der Initialisierungen %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Hole die Daten aus der Intialisierungsdatei

ini = evacuation_ini();

% Erstelle die Zufallsmatrix, die Ort und Geschwindigkeit gespeichert hat
rand_mat = feval('create_rand_mat',ini.d,ini.b,ini.l,ini.n); % Ist auch r0

% Erstelle die Geschwindigkeitsmatrix, Anfangsgeschwindigkeiten Null

vel_mat = zeros(ini.n,2);

% Bestimme die min. und max. x_Koordinate fr die Fluchttre

x_door_min = ((ini.b - ini.door) * ini.d ) / 2;

x_door_max = ((ini.b + ini.door) * ini.d ) / 2;

% Wandle ini.n in eine Zhlvariable um

n = ini.n;

%hold on

% Zeichne die Zufallspositionen
plot(rand_mat(:,1),rand_mat(:,2),'.');

% Hole die Kraftfelder, die die statische Anziehung modelliert
% (Komponentenweise)

[static_force_cell_x, static_force_cell_y] = feval('static_force', ini.d, ini.b, ini.l, ini.door, ini.c_1, ini.c_static)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%% Visualisierung %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

people_mat = rand_mat;

graph = plot(people_mat(:,1),people_mat(:,2),'.');
axis([0 ini.b 0 ini.l]);
axis square
grid off
title('Bewegung der Menschenmasse')
line([x_door_min,x_door_max],[ini.l,ini.l],'LineWidth',8,'Color','r')
set(graph,'MarkerSize',5);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%% LEAP-FROG-VERFAHREN %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%% Berechne v 1/2 und r1 als Anfangsschritt %%%%%%%%%%%%%%%%%%%%%%%


%Berechne, wieviele Personen sich in einem Feld befinden

field_mat = feval('count_field', people_mat, ini.n, ini.b, ini.l, ini.d);

% Berechne die Anziehungskraft der Menschen untereinander durch Gravitation
% (optional)

%[grav_force_x, grav_force_y] = feval('variable_force_grav', people_mat, ini.n, ini.c_2, ini.r_min);

% Berechne die Abstoungskraft der Menschen untereinander durch
% Lennard-Jones Potential

[lennard_force_x, lennard_force_y] = feval('variable_force_lennard', people_mat, ini.n, ini.eps, ini.sig_r, ini.r_min);

% Berechne die Kraft aus dem statischen Kraftfeld fr jede Person
% komponentenweise

[static_force_x, static_force_y] = feval('cell_to_people', static_force_cell_x, static_force_cell_y, people_mat, ini.d, ini.n);

% Berechne die Summe an Krften der einzelnen Komponenten

sum_force_x = static_force_x+ lennard_force_x;% + grav_force_x;

sum_force_y = static_force_y + lennard_force_y;% + grav_force_y;

% Bercksichtige den "Reibungsverlust" aufgrund der Dichte in der
% jeweiligen Zelle

[fin_force_x, fin_force_y] = feval('density', sum_force_x, sum_force_y, people_mat, field_mat, ini.d, ini.n);

% Berechne die Geschwindigkeit nach halben Zeitschritt aus den Krften v 1/2

vel_mat = feval('fin_vel_first', vel_mat, fin_force_x, fin_force_y, ini.n, ini.delta_t, ini.max_acc);

vel_mat

pause

% Berechne die neuen Koordinaten der Personen r1

for i = 1:ini.n
    
    people_mat(i,1) = people_mat(i,1) + 30 * vel_mat(i,1) * (ini.delta_t / 2);

    people_mat(i,2) = people_mat(i,2) + 30 * vel_mat(i,2) * (ini.delta_t / 2);
    
    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% KONTROLLBEDINGUNGEN !!!!!!! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[people_mat, vel_mat, n] = feval('control', people_mat, vel_mat, x_door_min, x_door_max, ini.b, ini.l, ini.d, ini.sig_r, n);

%%%%%%%%%%%%% Ende des Anfangsschrittes %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 



time_count = 0;

while n > 0
    
     time_count = time_count + 1;

%%%%%%%%%%%%% Berechnung der weiteren Schritte %%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Berechne, wieviele Personen sich in einem Feld befinden

field_mat = feval('count_field', people_mat, n, ini.b, ini.l, ini.d);

% Berechne die Anziehungskraft der Menschen untereinander durch Gravitation
% (optional)

%[grav_force_x, grav_force_y] = feval('variable_force_grav', people_mat, n, ini.c_2, ini.r_min);

% Berechne die Abstoungskraft der Menschen untereinander durch
% Lennard-Jones Potential

[lennard_force_x, lennard_force_y] = feval('variable_force_lennard', people_mat, n, ini.eps, ini.sig_r, ini.r_min);

% Berechne die Kraft aus dem statischen Kraftfeld fr jede Person
% komponentenweise

[static_force_x, static_force_y] = feval('cell_to_people', static_force_cell_x, static_force_cell_y, people_mat, ini.d, n);

% Berechne die Summe an Krften der einzelnen Komponenten

sum_force_x = static_force_x + lennard_force_x;% + grav_force_x;

sum_force_y = static_force_y + lennard_force_y;% + grav_force_y;


% Bercksichtige den "Reibungsverlust" aufgrund der Dichte in der
% jeweiligen Zelle

[fin_force_x, fin_force_y] = feval('density', sum_force_x, sum_force_y, people_mat, field_mat, ini.d, n);

% Berechne die Geschwindigkeit nach einem Zeitschritt aus den Krften 

delta_vel_mat = feval('fin_vel', vel_mat, fin_force_x, fin_force_y, n, ini.delta_t, ini.max_acc);

for i = 1:n
    
    for j = 1:2
        
    vel_mat(i,j) = vel_mat(i,j) + delta_vel_mat(i,j);       

    end
end

for i = 1:n
    
    norm_vel_mat(i) = norm(vel_mat(i,:));
    
    for j = 1:2
        
    vel_mat(i,j) = min( abs(vel_mat(i,j)), (ini.max_vel * abs(vel_mat(i,j)) ./ norm_vel_mat(i) ) ) .* sign(vel_mat(i,j));

    end
    
end

% Berechne die neuen Koordinaten der Personen r1

for i = 1:n
    
    people_mat(i,1) = people_mat(i,1) + vel_mat(i,1) * (ini.delta_t);

    people_mat(i,2) = people_mat(i,2) + vel_mat(i,2) * (ini.delta_t);
    
    
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% KONTROLLBEDINGUNGEN !!!!!!! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


[people_mat, vel_mat, n, m] = feval('control', people_mat, vel_mat, x_door_min, x_door_max, ini.b, ini.l, ini.d, ini.sig_r, n);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Zeichne die Zufallspositionen

    set(graph,'XData',people_mat(:,1),'YData',people_mat(:,2));
    
    M(time_count) = getframe;
 
% Speichere Vektor, der die Anzahl der evakuierten Personen wiedergibt und

 
evacuated(time_count) = m;
    
%%%%%%%%%%%%% Ende der weiteren Schritte %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

end



% Zeige ein Movie, das die Bewegung der Personen simuliert
pause 
movie(M,1,1)

% erzeuge einen Plot, der die evakuierten Personen graphisch darstellt
make_plot = feval('make_plot', evacuated);
newplot
plot(make_plot)

% Zeige an, wie lange die Evakuierung gedauert hat

disp(['Die Evakuierung dauerte ', num2str(time_count), ' Sekunden.'])

